#include "msp.h"
#include "osc.h"
#include "adc.h"
#include "osc.h"
#include "buttons.h"
#include "lcd.h"
#include "lcd_symbols.h"
#include "clock.h"
#include "timer.h"
#include "tasks.h"

#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#ifndef _CLOCK_C
#define _CLOCK_C

volatile uint16_t option;
volatile uint16_t botao;
unsigned int tempo = 0;

char dayConf[2];
char monthConf[2];
char yearConf[2];

char hourConf[2];
char minuteConf[2];
char secondConf[2];

/*HH:MM:SS*/
VAL_LCD LCDHOURS    = {0,1,1};
VAL_LCD LCDMINUTS   = {0,0,0};
VAL_LCD LCDSECONDS  = {0,0,0};

/*yy-mm-dd*/
VAL_LCD  LCDYEARS   = {1,4,14};
VAL_LCD  LCDMONTH   = {1,1,1};
VAL_LCD  LCDDAYS    = {1,1,1};

int Blink_pos[5]={1,4,7,10,13};

/*STATE VARIABLES*/
typedef enum {
       RTC_ADJ_DAY= 0,
       RTC_ADJ_MONTH,
       RTC_ADJ_YEAR,
       RTC_ADJ_HOURS,
       RTC_ADJ_MINUTES,
       RTC_ADJ_SECONDS,
       RTC_SAVE,
       RTC_CONVERTION
} RTC_ADJSTATES;   /*RTC STATE ADJUST*/

typedef struct {
    RTC_ADJSTATES state;
    int day;
    int month;
    int year;
    int hour;
    int minutes;
    int seconds;
    int save;
    int cancel;
} RTC_CTRL;        /*RTC CONTROL STATE*/

/*VARIABLES*/
volatile RTC_CTRL rtc; /*GLOBAL RTC VARIABLE*/


/*CLOCK CONFIGURATION TO 12 MHz*/
void clockConf(void){
    P2->OUT &= ~( BIT1 );
    P2->DIR |= BIT1 ;


    CS->KEY = 0x695A;                                                           /*UNLOCK CS MODULE DOR ACESS*/
    CS->CTL0 = 0;                                                               /*RESET TUNING PARAMETERS*/
    CS->CTL0 |= CS_CTL0_DCORSEL_2;                                              /*SET DCO TO 6 MHz (NOMINAL FREQUENCY BETWEEN  4 AND 8 MHz)*/
    CS->CTL1 |= CS_CTL1_SELA_3 | CS_CTL1_SELS_3 | CS_CTL1_SELM_3;               /*SELECT ACLK = REFO + SMCKL*/

    /*RTC CONFIGURATION*/
    RTC_C->CTL0 = RTC_C_KEY;                                                    /*UNLOCK RTC KEY PROTECTED REGISTERS*/
    RTC_C->PS1CTL = RTC_C_PS1CTL_RT1IP__64 |                                    /*SET DIVIDER T0 64*/ 
                    RTC_C_PS1CTL_RT1PSIE;
    RTC_C->CTL13 =  RTC_C_CTL13_HOLD |                                          /*HOLD THE RTC*/
                    RTC_C_CTL13_MODE |                                          /*SER RTC TO CALENDAR MODE*/ 
                    RTC_C_CTL13_BCD;

    RTC_C->CTL13 = RTC_C->CTL13 & ~(RTC_C_CTL13_HOLD);

    RTC_C->CTL0 = RTC_C->CTL0 & ~(RTC_C_CTL0_KEY_MASK);                         /*LOCK RTC KEY PROTECT REGISTERS*/

    __enable_irq();

    NVIC->ISER[0] = 1 << ((RTC_C_IRQn) & 31);

    SCB->SCR |= SCB_SCR_SLEEPONEXIT_Msk;

    userDefineTime();
}

/*RTC STATE MACHINE CONTROL*/
bool RTC_Task(void){

    if (ADC_GetButton() == _BTN_SETUP) {
          if (++rtc.state > RTC_SAVE) {

          }

    }
    if(rtc.state != RTC_SAVE){
        rtc_ShowClock();
    }

    switch(rtc.state){
        /*DATE*/


        case RTC_ADJ_DAY:
            rtc_setDays();
            character_Blink(RTC_ADJ_DAY);
            updateLcdTimeValues();

        break;

        case RTC_ADJ_MONTH:
            rtc_setMonths();
            character_Blink(RTC_ADJ_MONTH);
            updateLcdTimeValues();

            /*PASS LAST PARAMETER CONFIGURATION*/
            BCD_Number day_result;
            day_result = dec2bcd(rtc.day);
            sprintf(dayConf, "0x%x\n",*((int *)day_result.value));
            LCD_SetText(dayConf, 0, 0);

        break;

        case RTC_ADJ_YEAR:
            rtc_setYears();
            character_Blink(RTC_ADJ_YEAR);
            updateLcdTimeValues();

            /*PASS LAST PARAMETER CONFIGURATION*/
            BCD_Number month_result;
            month_result = dec2bcd(rtc.month);
            sprintf(monthConf, "0x%x\n",*((int *)month_result.value));
            LCD_SetText(monthConf, 0, 0);
        break;

        /*TIME*/

        case RTC_ADJ_HOURS:
            rtc_setHours();
            character_Blink(RTC_ADJ_HOURS);
            updateLcdTimeValues();

            /*PASS LAST PARAMETER CONFIGURATION*/
            BCD_Number year_result;
            year_result = dec2bcd(rtc.year);
            sprintf(yearConf, "0x%x\n",*((int *)year_result.value));
            LCD_SetText(yearConf, 0, 0);

        break;

        case RTC_ADJ_MINUTES:
            rtc_setMinutes();
            character_Blink(RTC_ADJ_MINUTES);
            updateLcdTimeValues();

            /*PASS LAST PARAMETER CONFIGURATION*/
            BCD_Number hour_result;
            hour_result = dec2bcd(rtc.hour);
            sprintf(hourConf, "0x%x\n",*((int *)hour_result.value));
            LCD_SetText(hourConf, 0, 0);

        break;

        case RTC_ADJ_SECONDS:
            /*PASS VARIABLE TO */
            rtc.state = RTC_SAVE;
            /*CLEAR LCD*/
            LCD_Clear();

            /*PASS LAST PARAMETER CONFIGURATION*/
            BCD_Number minute_result;
            minute_result = dec2bcd(rtc.minutes);
            sprintf(minuteConf, "0x%x\n",*((int *)minute_result.value));
            LCD_SetText(minuteConf, 0, 0);
            /*PASS LAST PARAMETER CONFIGURATION*/
            BCD_Number second_result;
            second_result = dec2bcd(0);
            sprintf(secondConf, "0x%x0\n",*((int *)second_result.value));
            LCD_SetText(secondConf, 6, 0);

        break;

        case RTC_SAVE:


           /*SAVE DEFINITIONS?*/
           if(RTC_AskToSave() == true & option == 2){
               /*CLEAR LCD CHARACTERS*/
               LCD_Clear();
               /*VERIFY IF DATA IS STORED IN SDCARD OR BLUETOOTH*/
               if(DEV_AquisitionStore() == true & option == 2){
                   /*DATA AQUISITION TO SDCARD*/
                   rtc.state = RTC_CONVERTION;

               }else if(DEV_AquisitionStore() == true & option == 3){
                   /*DATA AQUISITION TO BLUETOOTH*/
                   rtc.state = RTC_CONVERTION;
               }


           }else if(RTC_AskToSave()==true & option == 3){
               rtc.state = RTC_ADJ_DAY;
               LCD_Clear();
           }
        break;

        case RTC_CONVERTION:
            /*CLOCK CONVERTION*/
            /*TMP006 AQUISITION*/
            /*ADS1292R AQUISITION*/
            /*BATTERY READER*/
        break;
    }

    return false;
}
/*RTC SAVE USER CONFIGURATION*/
bool RTC_AskToSave(void){
    option = ADC_GetButton();
    //           1234567890123456
    LCD_SetText("  SAVE CONFIG?  ", 0, 0);
    LCD_SetText(" YES         NO ", 0, 1);

    switch(option){
    case _BTN_UP:
        /*ASK FOR WITCH DEVICE SAVING*/
        if(option == 2){
            return true;
        }
        LCD_Clear();

        break;
    case _BTN_DWN:
        /*START CONFIGURATION*/
        if(option == 3){
            return true;
        }
        LCD_Clear();

    break;

   }

    return false;
}

bool DEV_AquisitionStore(void){
    option = ADC_GetButton();
    //           01234567890123456
    LCD_SetText(" DEVICE TO STORE ", 0, 0);
    LCD_SetText(" SDC         BLE ", 0, 1);

    switch(option){
    case _BTN_UP:
        /*ASK FOR WITCH DEVICE SAVING*/
        if(option == 2){
            return true;
        }
        break;
    case _BTN_DWN:
        /*START CONFIGURATION*/
        if(option == 3){
            return true;
        }
    break;

   }

    return false;
}

void rtc_ShowClock(void){

    static bool blink = false;
    static uint16_t counter = 0;
    updateLcdTimeValues();

    //           01234567890123456

    //LCD_SetText("dd-mm-yy HH:MM", 1, 0);
    LCD_SetIntWithZero(LCDDAYS.valor, 1, 1);
    LCD_SetText("-", 3, 1);
    LCD_SetIntWithZero(LCDMONTH.valor, 4, 1);
    LCD_SetText("-", 6, 1);
    LCD_SetIntWithZero(LCDYEARS.valor,7, 1);

    //            01234567890123456
    LCD_SetIntWithZero(LCDHOURS.valor,   10, 1);
    LCD_SetIntWithZero(LCDMINUTS.valor,  13, 1);

    if(++counter > 5){
        if (blink == true) {
            LCD_SetText(":", 12, 1);
        }
        else {
            LCD_SetText(" ", 12, 1);
        }

        blink ^= true;

        counter = 0;
    }

}

void character_Blink(int state){
     static bool set = false;
     static uint16_t count = 0;
     /*LCD_SetInt(Blink_pos[state], 0, 0);*/
     if(++count > 2){
        if(set == true){

        }else{
           LCD_SetText("  ", Blink_pos[state],1);
        }
        set ^= true;
        count = 0;
     }
}



void CLK_GetTempo(int comando){
  tempo^=1;
}


    /*RTC USER SET MINUTES*/
    void rtc_setMinutes(void){
        /* INCREMENTO */
        if(ADC_GetButton() == _BTN_UP){
            if(rtc.minutes < MAX_MM){
                rtc.minutes++;
            }else{
                rtc.minutes = MIN_MM;
            }

        }
        /* DECREMENTO */
        if(ADC_GetButton()== _BTN_DWN){
            if(rtc.minutes > MAX_MM){
                rtc.minutes--;
            }else{
                rtc.minutes = MAX_MM;
            }

        }

    }

    /*RTC USER SET HOURS*/
    void rtc_setHours(void){
        /* INCREMENTO */
        if(ADC_GetButton() == _BTN_UP){
            if(rtc.hour < MAX_HH){
                rtc.hour++;
            }else{
                rtc.hour = MIN_HH;
            }

        }
        /* DECREMENTO */
        if(ADC_GetButton()== _BTN_DWN){
            if(rtc.hour > MIN_HH){
                rtc.hour--;
            }else{
                rtc.hour = MAX_HH;
            }

        }

    }

    /*RTC USER SET YEARS*/
    void rtc_setYears(void){
        /* INCREMENTO */
        if(ADC_GetButton() == _BTN_UP){
            if(rtc.year < MAX_YY){
                rtc.year++;
            }else{
                rtc.year = MIN_YY;
            }

        }
        /* DECREMENTO */
        if(ADC_GetButton() == _BTN_DWN){
            if(rtc.year > MIN_YY){
                rtc.year--;
            }else{
                rtc.year = MAX_YY;
            }

        }

    }

    /*RTC USER SET MONTHS*/
    void rtc_setMonths(void){
        /* INCREMENTO */
        if(ADC_GetButton() == _BTN_UP){
            if(rtc.month < MAX_MN){
                rtc.month++;
            }else{
                rtc.month = MIN_MN;
            }

        }
        /* DECREMENTO */
        if(ADC_GetButton() == _BTN_DWN){
            if(rtc.month > MIN_MN){
                rtc.month--;
            }else{
                rtc.month = MAX_MN;
            }

        }

    }

    /*RTC USER SET DAYS*/
    void rtc_setDays(void){
        /* INCREMENTO */
        if(ADC_GetButton() == _BTN_UP){
            if(rtc.day < MAX_DA){
                rtc.day++;
            }else{
                rtc.day = MIN_DA;
            }

        }
        /* DECREMENTO */
        if(ADC_GetButton() == _BTN_DWN){
            if(rtc.day > MIN_DA){
                rtc.day--;
            }else{
                rtc.day = MAX_DA;
            }

        }


    }

/*PICK USER KEY BUTTON DATA/TIME DEFENITION*/
void userDefineTime(void){

  RTCYEAR = yearConf;
  RTCMON  = monthConf;
  RTCDAY  = dayConf;
  RTCHOUR = hourConf;
  RTCMIN  = minuteConf;
  RTCSEC  = secondConf;
}

/**/
void updateLcdTimeValues(void){
  LCDYEARS.valor   = rtc.year;                  /*YEAR   = 0xYYYY*/
  LCDMONTH.valor   = rtc.month;                 /*MOUNTH = 0xMM*/
  LCDDAYS.valor    = rtc.day;                   /*DAY    = 0xDD*/
  LCDHOURS.valor   = rtc.hour;                  /*HOUR   = 0xhh*/
  LCDMINUTS.valor  = rtc.minutes;               /*MIN    = 0xmm*/
  LCDSECONDS.valor = rtc.seconds;               /*SEC    = 0xss*/
}


BCD_Number dec2bcd(int bin_number){
    BCD_Number bcd_number;
    unsigned int i;

    for(i = 0;i<sizeof(bcd_number.value);i++){
        bcd_number.value[i] = bin_number %10;
        bin_number = bin_number/10;
        bcd_number.value[i] |=bin_number%10 <<4;
        bin_number= bin_number/10;
    }

    return bcd_number;
}


// RTC interrupt service routine
void RTC_C_IRQHandler(void){
    if (RTC_C->PS1CTL & RTC_C_PS1CTL_RT1PSIFG){
        P2OUT ^= BIT1; //
        // Clear the pre-scalar timer interrupt flag
        RTC_C->PS1CTL &= ~(RTC_C_PS1CTL_RT1PSIFG);
    }
}


#endif // _CLOCK_C
